﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Problems.LinearOrdering
{
    /// <summary>
    /// An operator which evaluates LOP solutions given in matrix representation.
    /// </summary>
    [Item("SuperdiagonalMatrixTriangulationEvaluator", "An operator which evaluates LOP solutions given in matrix representation.")]
    [StorableClass]
    public sealed class SuperdiagonalMatrixTriangulationEvaluator : MatrixTriangulationEvaluator
    {
        [StorableConstructor]
        private SuperdiagonalMatrixTriangulationEvaluator(bool deserializing) : base(deserializing) { }
        private SuperdiagonalMatrixTriangulationEvaluator(SuperdiagonalMatrixTriangulationEvaluator original, Cloner cloner) : base(original, cloner) { }
        public SuperdiagonalMatrixTriangulationEvaluator() : base() { }

        public override IDeepCloneable Clone(Cloner cloner)
        {
            return new SuperdiagonalMatrixTriangulationEvaluator(this, cloner);
        }

        protected override double CalculateUpperTriangle(DoubleMatrix m, Permutation p)
        {
            double sum = 0;
            for (int i = 1; i < m.Columns; i++)
            {
                for (int j = 0; j < i; j++)
                {
                    sum += m[p[j], p[i]];
                }
            }

            return sum;
        }
    }
}
